(use eggdoc htmlprag sxml-tools)

(define (pull-demo-code-in)
  `(pre ,(with-input-from-file "amb-demo.scm" read-string)))

(define doc
  `((eggdoc:begin
     (name "amb")
     (description (p "The non-deterministic backtracking ambivalence operator"))
     (author (url "http://www.chust.org/" "Thomas Chust"))
     (history
      (version "1.2.0" "Non-determinism now optional, better controllability of scope")
      (version "1.1.1" "Fixed bug in amb.setup [Thanks to Kon Lovett]")
      (version "1.1.0" "Renamed " (tt "bag-of") " to " (tt "amb-collect") " and added " (tt "amb-assert"))
      (version "1.0.0" "Initial release"))
     
     (usage)
     (download "amb.egg")
     
     (documentation
      (p "The " (tt "amb") " operator is a nice toy and sometimes a useful tool for lightweight logic programming. Its implementation is also a good exercise in the handling of continuations.")

      (subsection
       "Normal interface"
       (group
	(macro
	 "(amb expr ...) => <top>"
	 (p "In the form " (tt "(amb)") " the expression always fails.")
	 (p "If the expression has any parameters, the first one of them is evaluated and the result is returned. If a subsequent occurrence of " (tt "amb") " fails, though, backtracking may cause the second of the given expressions to be selected for evaluation, then the third and so forth until the whole program does not fail if at all possible.")
	 (p "The backtracking mechanism is described along with the " (tt "amb-failure-continuation") " parameter below."))
	(macro
	 "(amb/random expr ...) => <top>"
	 (p "Works like " (tt "amb") " but the parameters are not selected in sequence but randomly. None of them is selected more than once, though."))
	(macro
	 "(amb-find expr #!optional failure-value) => <top>"
	 (p "Evaluates " (tt "expr") " returning its value if successful (possibly after backtracking).")
	 (p "If " (tt "expr") " cannot be evaluated successfully and the expression tree is exhausted, " (tt "failure-value") " is evaluated and the result is returned instead. If no " (tt "failure-value") " is specified, an exception occurs. See the " (tt "amb-failure-continuation") " parameter below for a description of the exception."))
	(macro
	 "(amb-collect expr) => <list>"
	 (p "Evaluates " (tt "expr") " and performs backtracking repeatedly until all possible values for it have been accumulated in a list, which is returned."))
	(macro
	 "(amb-assert ok?) => <void>"
	 (p "Evaluates " (tt "ok?") " and fails if it is " (tt "#f") "."))))

      (subsection
       "Additional stuff exported by amb-base"
       (group
	(definition
	  (signatures
	   (signature "parameter" "amb-failure-continuation"))
	  (p "Seen in a global context, the " (tt "amb") " operator transforms the whole program that contains it into a depth first search for return values from " (tt "amb") " forms that will not cause failure.")
	  (p "This is realized using a backtracking system that invokes previously stored continuations whenever an " (tt "amb") " expression fails. The " (tt "amb-failure-continuation") " parameter is the status variable for this system.")
	  (p "At the start of the program, or when no further backtracking options are available, this is set to a procedure of no arguments that throws an exception of the composite " (tt "(exn amb)") " kind (except when a " (tt "amb-collect") " statement is being processed, where the parameter will point to a procedure signalling " (tt "amb-collect") " that there are no more backtracking options available).")
	  (p "In all other cases this parameter is set to a procedure of no arguments that causes backtracking to the next possible alternative in the \"tree\".")
	  (p "If you want to restrict the scope of backtracking to something smaller than the whole past program, use " (tt "amb-find") " or " (tt "amb-collect") " which restore this parameter to its original value when they are done evaluating the expressions they were given."))
	(procedure
	 "(amb-thunks <list of procedures>) => <top>"
	 (p "The backend of " (tt "amb") ". " (tt "amb") " wraps all its parameters into thunks and passes a list of them into this procedure, " (tt "amb/random") " shuffles the list first."))
	(procedure
	 "(amb-find-thunk <procedure> #!optional <procedure>) => <top>"
	 (p "The backend of " (tt "amb-find") ". " (tt "amb-find") " wraps its parameters into thunks and passes them into this procedure."))
	(procedure
	 "(amb-collect-thunk <procedure>) => <list>"
	 (p "The backend of " (tt "amb-collect") ". " (tt "amb-collect") " wraps its parameter into a thunk and passes it into this procedure.")))))

      (examples
       (p "The following code is a rewrite of an example from the book \"Teach Yourself Scheme in Fixnum Days\" by Dorai Sitaram. The book gives the following problem setting:")
       (blockquote
	(p "The Kalotans are a tribe with a peculiar quirk. Their males always tell the truth. Their females never make two consecutive true statements, or two consecutive untrue statements.")
	(p "An anthropologist (let's call him Worf) has begun to study them. Worf does not yet know the Kalotan language. One day, he meets a Kalotan (heterosexual) couple and their child Kibi. Worf asks Kibi: \"Are you a boy?\" Kibi answers in Kalotan, which of course Worf doesn't understand.")
	(p "Worf turns to the parents (who know English) for explanation. One of them says: \"Kibi said: 'I am a boy.'\" The other adds: \"Kibi is a girl. Kibi lied.\"")
	(p "Solve for the sex of the parents and Kibi."))

       (p "So here is the solution:")
       ,(pull-demo-code-in))

      (license
       "Copyright (c) 2005, Thomas Chust <chust@web.de>.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

  Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer. Redistributions in
  binary form must reproduce the above copyright notice, this list of
  conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution. Neither the name of the
  author nor the names of its contributors may be used to endorse or
  promote products derived from this software without specific prior
  written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS \"AS
IS\" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."))))

(eggdoc->html doc (eggdoc:make-stylesheet doc))
